//An Igor procedure to determine the travel time of ultrasonic waves using waveform files (.csv) recorded by a digital oscilloscope
//This procedure is suitable for the calculation of travel time between two phases that have a 180 degree phase shift. For phases that do not have a phase shift, use the "Travel_Time_In_Phase_2.ipf" procedure instead.
//
//Author: Zhicheng Jing
//Address: Department of Earth and Space Sciences, Southern University of Science and Technology, 1088 Xueyuan Blvd, Shenzhen, Guangdong, 518055, China
//Email: jingzc@sustech.edu.cn
//
//Modified on 01/20/2017
//Modified on 11/15/2017 to deal with the csv data generated by the MSO54 oscilloscope
//Modified on 04/06/2019

#pragma rtGlobals=1		// Use modern global access method.
function ttop2()

//for MSO54 oscilloscope data
LoadWave/J/D/O/E=1/K=0/B="F=0,N=Data_t; F=0,N=Data_A;"
Wave Data_A, Data_t
variable t_step=Data_A[3]
DeletePoints 0, 9, Data_t
DeletePoints 0, 9, Data_A
Redimension /D Data_t
Redimension /D Data_A
Display Data_A vs Data_t
ShowInfo

//for DPO5104B oscilloscope data
//LoadWave/J/D/O/E=1/K=0/B="N='_skip_'; N=Info; N='_skip_'; N=Data_t; N=Data_A;"
//Wave Info,Data_A, Data_t
//variable t_step=Info[1]
//Display Data_A vs Data_t
//ShowInfo

DoWindow/F Graph0
NewPanel /K=2 /W=(187,368,437,531) as "Pause for Cursor"
DoWindow/C tmp_PauseforCursor
AutoPositionWindow/E/M=1/R=Graph0
DrawText 21,20,"Mark the beginning and the end of the data"
DrawText 21,40,"Click Continue."
Button button0,pos={80,58},size={92,20},title="Continue"
Button button0,proc=UserCursorAdjust_ContButtonProc	
PauseForUser tmp_PauseforCursor,Graph0
variable D_start=pcsr(A),D_end=pcsr(B)
if(mod(D_end-D_start,2)==0)
D_end-=1
endif
variable N_Data=D_end-D_start+1

variable A_mean
Duplicate/R=[D_start,D_end] Data_A Data_A2,A_offset
Duplicate/R=[D_start,D_end] Data_t Data_t2
A_mean=mean(Data_A2)
A_offset-=A_mean

FFT/DEST=A_fft A_offset
Make/N=(N_Data/2+1)/D A_fft_freq
MatrixOp A_fft_mag=mag(A_fft)
variable t
for(t=0;t<N_Data/2+1;t+=1)
A_fft_freq[t]=t/N_Data/t_step
endfor

Display A_fft_mag vs A_fft_freq
Variable f_cutoff_L=8e6, f_cutoff_H=70e6,N_cutoff_L,N_cutoff_H
N_cutoff_L=f_cutoff_L*N_Data*t_step
N_cutoff_H=f_cutoff_H*N_Data*t_step
Duplicate A_fft_mag A_fft_mag_cut
DeletePoints N_cutoff_H,N_Data/2+1-N_cutoff_H, A_fft,A_fft_mag_cut
InsertPoints N_cutoff_H, N_Data/2+1-N_cutoff_H, A_fft,A_fft_mag_cut
DeletePoints 0,N_cutoff_L, A_fft,A_fft_mag_cut
InsertPoints 0, N_cutoff_L, A_fft,A_fft_mag_cut
AppendtoGraph A_fft_mag_cut vs A_fft_freq; ModifyGraph rgb(A_fft_mag_cut)=(0,0,65535)
IFFT/DEST=A_offset A_fft
Display Data_A2 vs Data_t2; AppendtoGraph A_offset vs Data_t2;ModifyGraph rgb(A_offset)=(0,0,65535)
ShowInfo

DoWindow/F Graph2
NewPanel /K=2 /W=(187,368,437,531) as "Pause for Cursor"
DoWindow/C tmp_PauseforCursor
AutoPositionWindow/E/M=1/R=Graph2
DrawText 21,20,"Mark the beginning and the end of the buffer rod echo"
DrawText 21,40,"Click Continue."
Button button0,pos={80,58},size={92,20},title="Continue"
Button button0,proc=UserCursorAdjust_ContButtonProc	
PauseForUser tmp_PauseforCursor,Graph2
variable B_start=pcsr(A),B_end=pcsr(B)

variable N_B, N_S, t_B, t_S, tt_S
Duplicate A_offset S_A
DeletePoints 0, B_start, S_A
InsertPoints 0, B_start, S_A
DeletePoints B_end, N_Data-B_end, S_A
InsertPoints B_end, N_Data-B_end, S_A
Display S_A vs Data_t2

variable A_corr_max,A_S_min,t_S_min
Duplicate A_offset, A_offset_corr
Correlate S_A, A_offset_corr
A_corr_max=A_offset_corr[N_Data-1]
Display A_offset_corr
ShowInfo

DoWindow/F Graph4

NewPanel /K=2 /W=(187,368,437,531) as "Pause for Cursor"
DoWindow/C tmp_PauseforCursor
AutoPositionWindow/E/M=1/R=Graph4
DrawText 21,20,"Mark the trough for the sample echo"
DrawText 21,40,"Click Continue."
Button button0,pos={80,58},size={92,20},title="Continue"
Button button0,proc=UserCursorAdjust_ContButtonProc	
PauseForUser tmp_PauseforCursor,Graph4

N_S=pcsr(A)

Duplicate/R=[N_S-20, N_S+20] A_offset_corr S_corr
WaveStats S_corr
t_S=V_minloc
A_S_min=V_min

Make/N=1 t_B_S={t_S},A_B_S={A_S_min}
AppendtoGraph A_B_S vs t_B_S;ModifyGraph mode(A_B_S)=3,msize(A_B_S)=5, rgb(A_B_S)=(0,0,65535)

tt_S=t_S*t_step
Print "Two-way travel time in sample: ",tt_S, "s"

Duplicate/R=[B_start, B_end+t_S*2] A_offset A_AV
Duplicate/R=[B_start, B_end+t_S*2] Data_t2 t_AV
Duplicate/R=[B_start, B_end] A_offset A_SP
Duplicate/R=[B_start+t_S, B_end+t_S] Data_t2 t_SP

A_SP=A_SP*A_S_min/A_corr_max

Display A_AV vs t_AV; AppendtoGraph A_SP vs t_SP
ModifyGraph rgb(A_SP)=(0,0,65535)
end
